@@ -1,24 +1,33 @@
 import asyncio
+import logging
 import os
 from contextlib import asynccontextmanager, contextmanager
 
-from stagehand import Stagehand, StagehandConfig, StagehandPage
-from stagehand.context import StagehandContext
+from playwright.async_api import (
+    async_playwright,
+    BrowserContext,
+    Page as PlaywrightPage,
+)
 
 from .page import Page as VibePage
-from ..automation import Automation
+
+logger = logging.getLogger(__name__)
 
 
 class WrappedContext:
     """Browser context wrapper that automatically wraps pages with VibePage functionality."""
 
     def __init__(
-        self, stagehand_context: StagehandContext, stagehand_pages: list[StagehandPage]
+        self,
+        playwright_context: BrowserContext,
+        playwright_pages: list[PlaywrightPage] = None,
     ):
-        self._stagehand_context = stagehand_context
+        """Initialize WrappedContext with native Playwright objects."""
+        self._playwright_context = playwright_context
         self._pages = []
-        for stagehand_page in stagehand_pages:
-            self._pages.append(VibePage(stagehand_page))
+        if playwright_pages:
+            for page in playwright_pages:
+                self._pages.append(VibePage(page))
 
     @property
     def pages(self):
@@ -30,31 +39,17 @@ class WrappedContext:
             await page._wait_for_login_task()
 
     def __getattr__(self, name):
-        # Forward attribute lookups to the underlying StagehandContext
-        attr = getattr(self._stagehand_context, name)
+        # Forward attribute lookups to the underlying Playwright context
+        attr = getattr(self._playwright_context, name)
 
         # Special handling for methods that return pages
         if name == "new_page":
             # Replace with our own implementation that wraps the page
             async def wrapped_new_page(*args, **kwargs):
-                stagehand_page = await self._stagehand_context.new_page(*args, **kwargs)
-                vibe_page = VibePage(stagehand_page)
-                # Register default login handler upon page creation
-                vibe_page.on("login_required")
-
-                # check whether we are under workflow development by checking if the EXECUTION_ID has a `test-` prefix
-                # see https://github.com/orby-ai-engineering/vibe-automation-server/blob/bafc4268ab0e61c7ef3f77c46c7f883f1ae38be4/packages/web_server/web_server/message_processing/handlers/flow.py#L71-L73
-                try:
-                    is_test_mode = Automation.get_instance().execution_id.startswith(
-                        "test-"
-                    )
-                except RuntimeError:
-                    is_test_mode = False
-
-                # enable checkpoint review in test mode for all pages by default
-                if is_test_mode:
-                    vibe_page.enable_checkpoint_review()
-
+                playwright_page = await self._playwright_context.new_page(
+                    *args, **kwargs
+                )
+                vibe_page = VibePage(playwright_page)
                 self._pages.append(vibe_page)
                 return vibe_page
 
@@ -90,14 +85,14 @@ def get_browser_sync(headless: bool | None = None, slow_mo: float | None = None)
     when new_page() is called, eliminating the need for manual wrap() calls.
     """
     try:
-        wrapped_context, stagehand = asyncio.run(
+        wrapped_context, browser = asyncio.run(
             create_browser_context_async(headless, slow_mo)
         )
         yield wrapped_context
     finally:
         # Wait for any background login tasks before closing
         asyncio.run(wrapped_context._wait_for_login_tasks())
-        asyncio.run(stagehand.close())
+        asyncio.run(browser.close())
 
 
 @asynccontextmanager
@@ -107,47 +102,50 @@ async def get_browser_context(
     """Async version of get_browser for use in async contexts."""
 
     try:
-        wrapped_context, stagehand = await create_browser_context_async(
-            headless, slow_mo
-        )
+        wrapped_context, browser = await create_browser_context_async(headless, slow_mo)
         yield wrapped_context
     finally:
         # Wait for any background login tasks before closing
         await wrapped_context._wait_for_login_tasks()
-        await stagehand.close()
+        await browser.close()
 
 
 async def create_browser_context_async(
     headless: bool | None = None, slow_mo: float | None = None
 ):
-    """Create a browser context"""
+    """Create a browser context using native Playwright."""
+    logger.info("Creating native Playwright browser context...")
+
+    playwright = await async_playwright().start()
+
     connection_url = os.environ.get("CONNECTION_URL")
     if connection_url:
         # Connect to existing browser instance via CDP
-        local_browser_launch_options = {
-            "cdp_url": connection_url,
-        }
+        logger.info(f"Connecting to existing browser via CDP: {connection_url}")
+        browser = await playwright.chromium.connect_over_cdp(connection_url)
+        # Get the default context from the connected browser
+        contexts = browser.contexts
+        if contexts:
+            context = contexts[0]
+        else:
+            context = await browser.new_context()
     else:
         # Launch a new browser instance
-        local_browser_launch_options = {
-            "headless": headless,
-            "slow_mo": slow_mo,
+        logger.info("Launching new browser instance")
+        browser = await playwright.chromium.launch(
+            headless=headless,
+            slow_mo=slow_mo,
             # scale ratio is set to 1 to ensure coordinates are calculated correctly
-            "args": ["--force-device-scale-factor=1"],
-        }
+            args=["--force-device-scale-factor=1"],
+        )
+        context = await browser.new_context()
 
-    config = StagehandConfig(
-        env="LOCAL",
-        model_name="claude-3-7-sonnet-20250219",
-        model_client_options={"apiKey": os.getenv("ANTHROPIC_API_KEY")},
-        local_browser_launch_options=local_browser_launch_options,
-    )
+    # Set default timeout
+    context.set_default_timeout(3000)
 
-    stagehand = Stagehand(config)
-    await stagehand.init()
-    stagehand.context.set_default_timeout(3000)
-    stagehand_pages = await stagehand.context.pages()
+    # Get existing pages and wrap them
+    pages = context.pages
+    wrapped_context = WrappedContext(context, pages)
 
-    wrapped_context = WrappedContext(stagehand.context, stagehand_pages)
-
-    return wrapped_context, stagehand
+    logger.info("Native Playwright browser context created successfully")
+    return wrapped_context, browser
